;Script to automate the BrainVoyagerQX Granger Causality Mapping plugin.
;Jasper van den Bosch, march 2009, www.crossmodal.de
;Tested on Version 1.10.2.1198
;BrainVoyager is Copyright  2001-2009 Rainer Goebel

;necessary gui-properties for interaction obtained with AuoIT v3 Window Info Tool:

;selection-form
; title=" BrainVoyager QX Plugin"
; class="QWidget"
; default-focus="selection-text-control"
; interaction "ENTER"->confirm selection

;openfile-dialog
; title=variable
; Class=#32770
; default-focus="filename field"
; interaction "ALT-F"->confirm selection
; interaction handle="0x00470692"/tab="3rd"->cancel

;savefile-dialog
; title=variable
; Class=#32770
; default-focus="filename field"
; interaction "ALT-S"->confirm selection
; interaction tab="3rd"->cancel
;IniRead ( "filename", "section", "key", "default" )

#Include <Date.au3>
#include <Debug.au3>
#Include "bvqxfiles.au3"

;CONFIGURATION
_DebugSetup ("autoGCM_debug")
Opt('MustDeclareVars', 1)
Opt("SendKeyDelay",1) 
Opt("SendKeyDownDelay",1) 
;DllCall("kernel32.dll", "int", "Wow64DisableWow64FsRedirection", "int", 1)


;SETTINGS
Const $waitTimeOut = 15	;time to wait in sec. for a window to show up; to prevent the script from running indefinitely.
Const $errorMsgTitle = "autoGCM Error"
Const $gcmPrefix = "autoGCM"

;BRAINVOYAGER CONSTANTS
Const $winLinkVTC = " Link 3D Volume Time Course (VTC)" ;title of link VTC dialog window
Const $winVOIanalysis = " Volume-Of-Interest Analysis"	;title of VOI analysis window
Const $mnuPluginGCMpos = 6 						;position of GCM plugin in plugins menu
Const $winGCM = " BrainVoyager QX Plugin"			;title of GCM plugin windows
Const $winGCMvtcPRE = "Add a VTC for condition: "
Const $winGCMvtcPOST = " (cancel to stop adding)"
Const $winGCMprt = "Specify a protocol file with VOLUME time-resolution"
Const $winGCMsave = "Choose path and name-prefix for results:"
Const $fileOpenEditControlID = 1148		;the id of the main edit control in open file dialogs to enter the filename
Const $gcmEditControlID = "[CLASS:QWidget; INSTANCE:4]"	;Control "ID" of the textedit in the GCM window
Const $maxPollsFinished = 20			;the number of times to poll the log for finish of script before continuing anyway
Const $intervalPollFinished = 60000		;interval between polls of log for plugin calculation finish

;VARIABLES
Dim $conditionName = "dummycond"
Dim $conditionNumber = 0
Dim $vtcNumber = 0
Dim $winGCMvtc = ""
Dim $datetime = ""
Dim $mdm[1]
Dim $vmr = ""
Dim $vtc = ""
Dim $prt = ""
Dim $voi = ""
Dim $dummyVTC = ""	;vtc file name
Dim $nVOIs = 0									;number of regions-of interest in de .voi file
Dim $gcm = ""
Dim $singleMDM = ""
Dim $nPollsFinished = 0
Dim $path
Dim $VTCPRT	;array with VTC-PRT combinations
Dim $conditions ;array with conditionnames in PRT
Dim $bvqx = "" ;path to brainvoyager executable
Dim $singleMDMnameOnly = ""
Dim $waitedSeconds = 0

$dateTime = getDateTimeString()

;INPUT: folder with necessary files:
$path = FileSelectFolder( "autoGCM: Select data folder (containing .vmr, .voi and .mdm).", "" )
If $path == "" Or @error Then
	MsgBox(1,$errorMsgTitle,"No folder specified, exiting autoGCM.")
EndIf
If Not (StringRight($path, 1) == "\") Then
	$path = $path & "\"
EndIf

;GET FILES
$vmr = fileFindName($path,"vmr")
If Not $vmr Then
	MsgBox(0,$errorMsgTitle,"No .vmr found.")
	Exit(1)
EndIF 
$voi = fileFindName($path,"voi")
If Not $voi Then
	MsgBox(0,$errorMsgTitle,"No voi found.")
	Exit(1)
EndIF 
$mdm = filesFindNames($path,"mdm")
If Not IsArray($mdm) Then
	MsgBox(0,$errorMsgTitle,"No .mdm found.")
	Exit(1)
EndIF 

;READ VOI
$nVOIs = readNumberFromVOI($path&$voi)
If @error == -1 Then
	MsgBox(0,$errorMsgTitle,"Number of VOIs not found in .voi file.")
	Exit(1)
ElseIf @error == -2 Then
	MsgBox(0,$errorMsgTitle,"VOI File indicates zero voi definitions.")
	Exit(1)
EndIf

;READ MDM
For $singleMDM in $mdm
	$VTCPRT = readMDM($path&$singleMDM)
	If @error == -1 Then
		MsgBox(0,$errorMsgTitle,"Could not open .mdm "&$singleMDM)
		Exit(1)
	ElseIf @error == -2 Then
		MsgBox(0,$errorMsgTitle,"No entries found in .mdm "&$singleMDM)
		Exit(1)
	ElseIf @error == -3 Then
		$prtLackingVTC = $VTCPRT[UBound($VTCPRT,1)-1][0]
		MsgBox(0,$errorMsgTitle,"A VTC is lacking a referenced protocol file: "&$prtLackingVTC)
		Exit(1)
	Else
		$conditions = readConditionsFromPRT($VTCPRT[0][1])
		If @error == -1 Then
			MsgBox(0,$errorMsgTitle,"Error opening .prt "&$VTCPRT[0][1]&" specified in .mdm: "&$singleMDM)
			Exit(1)
		ElseIf @error == -2 Then
			MsgBox(0,$errorMsgTitle,"No entries found in (referenced) .prt "&$VTCPRT[0][1]&" in .mdm "&$singleMDM )
			Exit(1)
		EndIf
	EndIf
	$dummyVTC = $VTCPRT[0][0]
Next

;START BRAINVOYAGER
$bvqx = @ProgramFilesDir&"\BrainVoyager QX\BrainVoyagerQX.exe"
If Not FileExists($bvqx) Then
	MsgBox(0,$errorMsgTitle,"Could not find BrainVoyagerQX installation.")
	Exit(1)
EndIf
Run($bvqx)

;ACCEPT DISCLAIMER
; title=" Disclaimer"
WinWaitActive(" Disclaimer","",$waitTimeOut)
Send("{ENTER}")

;OPEN VMR
Send("^o")
WinWaitActive("Open File","",$waitTimeOut)
ControlSetText("Open File", "", $fileOpenEditControlID, $path&$vmr )
Send("{ENTER}")
WinWait("3D Volume Tools","",$waitTimeOut)

;LINK dummy VTC
Send("^k")
WinWaitActive($winLinkVTC,"",$waitTimeOut)
ControlClick($winLinkVTC, "", "[CLASS:QWidget; INSTANCE:9]")
WinWaitActive("Select VTC File","",$waitTimeOut)
ControlSetText("Select VTC File", "", $fileOpenEditControlID, $dummyVTC )
Send("{ENTER}")
WinWaitClose("Select VTC File","",$waitTimeOut)
Send("{ENTER}")

;OPEN VOI
Send("+r")
WinWait($winVOIanalysis,"",$waitTimeOut)
ControlClick($winVOIanalysis, "", "[CLASS:QWidget; INSTANCE:6]")
WinWaitActive("Select VOI File","",$waitTimeOut)
ControlSetText("Select VOI File", "", $fileOpenEditControlID, $path&$voi )
Send("{ENTER}")
WinClose($winVOIanalysis)

; MDM loop
For $singleMDM in $mdm
	_DebugOut("[autoGCM] Starting GCM plugin for MDM: "&$singleMDM)
	$VTCPRT = readMDM($path&$singleMDM)

	;START PLUGIN
	Send("!p")
	Send("+{DOWN " & $mnuPluginGCMpos & "}")
	Send("{ENTER}")
	WinWaitActive($winGCM,"",$waitTimeOut)

	;SPECIFY VOIS
	;0 selection to stop selecting 
	Send(String($nVOIs+1))	;choose "select all" option which has a number one larger than the number of vois.
	Send("{ENTER}")
	WinWaitActive($winGCM,"",$waitTimeOut)

	;OPTION CONDITION-INTERVAL-SETTING
	;2 for .prt based
	Send("2")	;choose option 2, select conditions from protocol file..
	Sleep(200)
	Send("{ENTER}")

	;SPECIFY CONDITIONS
	; empty to stop
	For $conditionNumber = 0 to UBound($conditions)-1
		
		$conditionName = $conditions[$conditionNumber]
		WinWaitActive($winGCM,"",$waitTimeOut)	

		Send($conditionName)	
		Send("{ENTER}")
		$winGCMvtc = $winGCMvtcPRE & $conditionName & $winGCMvtcPOST		

		;SPECIFY VTCs
		; cancel to stop
		For $vtcNumber = 0 to UBound($VTCPRT,1)-1

			$vtc = $VTCPRT[$vtcNumber][0]
			$prt = $VTCPRT[$vtcNumber][1]
			
			;SELECT VTC
			WinWaitActive($winGCMvtc,"",$waitTimeOut)
			ControlSetText($winGCMvtc, "", $fileOpenEditControlID, $vtc)
			Send("{ENTER}")

			;SELECT CORRESPONDING PRT FILE
			WinWaitActive($winGCMprt,"",$waitTimeOut)
			ControlSetText($winGCMprt, "", $fileOpenEditControlID, $prt)
			Send("{ENTER}")

			;SELECT CORRESPONDING PRT CONDITION
			WinWaitActive($winGCM,"",$waitTimeOut)
			Send(String($conditionNumber+1))		
			Sleep(100)
			Send("{ENTER}")
			
		Next ;$vtcNumber
		
		;STOP ADDING VTCs
		WinWaitActive($winGCMvtc,"",$waitTimeOut)
		Send("{TAB 3}")				;move focus to cancel button
		Send("{ENTER}")
	Next ;$conditionNumber

	;STOP ADDING CONDITIONS
	WinWaitActive($winGCM,"",$waitTimeOut)
	Send("{ENTER}")

	WinWaitActive($winGCMsave,"",$waitTimeOut)
	;SAVE RESULT FILE
	$singleMDMnameOnly = pathGetFileName($singleMDM)
	$gcm = $path&$gcmPrefix&"_"&$datetime&"_"&$singleMDMnameOnly
	ControlSetText($winGCMsave, "", $fileOpenEditControlID, $gcm)
	Send("{ENTER}")
	
	_DebugOut("[autoGCM] Started plugin..")
	;start waiting for plugin to finish:
	$nPollsFinished = 0
	While $nPollsFinished < $maxPollsFinished
		$nPollsFinished += 1
		Sleep($intervalPollFinished)
		If pollLogPageGCMfinished() Then
			ExitLoop
		EndIf
	Wend
	$waitedSeconds = Round(($nPollsFinished*$intervalPollFinished)/1000)
	_DebugOut("[autoGCM] Waited "&$waitedSeconds&"s for GCM to finish, now continuing.")
	
Next ;next MDM or done

Sleep(60000) ;sleep a minute (to ensure all processes finished)

;EXIT BRAINVOYAGER
WinClose(" BrainVoyager QX")	;this frees up system resources.
