#include <Debug.au3>
#include <file.au3>
#include <array.au3>
_DebugSetup ("autoGCM_debug")

Func readMDM($path)  
	Const $startLine = 10
	Local $file
    Local $line
	Local $nEntries = 0
	Local $entry
	Local $VTCPRT[1][2]
	Local $prt = ""

	_DebugOut("[readMDM] reading.. ")
	$file = FileOpen ( $path, 0 )
	If $file == -1 Then
		_DebugOut("[readMDM] {-1} could not open file: "&$path)
		SetError(-1)
		Return $VTCPRT
	EndIf
	While 1
		$line = FileReadLine($file)
		If @error Then ExitLoop	;no more lines
		_DebugOut("[readMDM] line: "&$line)
		$entry = StringRegExp($line, '\"([^"]*)\"', 3)
		If @error == 0 Then
			;found a vtc-rtc combination
			$nEntries += 1
			_DebugOut("[readMDM] entry number: "&$nEntries&" contains: "&_ArrayToString($entry,@TAB,0,1))
			ReDim $VTCPRT[$nEntries][2]
			$VTCPRT[$nEntries-1][0] = slashesToBackslashes($entry[0])
			$prt = readPRTfromVTC($entry[0])
			If $prt == "" Or $prt =="<none>" Then
				_DebugOut("[readMDM] {-3} entry number: "&$nEntries&" no prt found.")
				SetError(-3)
				Return $VTCPRT
			EndIf
			_DebugOut("[readMDM] entry number: "&$nEntries&" prt found: ["&$prt&"]")
			$VTCPRT[$nEntries-1][1] = slashesToBackslashes($prt)
			
		EndIf
	Wend
	If $nEntries == 0 Then
		_DebugOut("[readMDM] {-2} No entries found.")
		SetError(-2)
		Return $VTCPRT
	EndIf
	FileClose ( $file )
	_DebugOut("[readMDM] exit.. ")
	Return $VTCPRT
EndFunc

Func slashesToBackslashes($path)
	Local $newPath
	$newPath = StringReplace($path, "/", "\")

	Return $newPath
EndFunc

;Reads condition names design matrix file, either .prt or .rtc
Func readConditionsFromPRT($path)  
	Const $lastLineHeader = 15
	Local $file
    Local $line
	Local $nEntries = 0
	Local $conditions[1]
	Local $lastLineEmpty = False
	_DebugOut ("[readConditionsFromPRT] reading...")
	$file = FileOpen ( $path, 0 )
	If $file == -1 Then
		_DebugOut("[readConditionsFromPRT] {-1} could not open file: "&$path)
		SetError(-1)
		Return $conditions
	EndIf
	;skip to last line containing header info
	$line = FileReadLine($file, $lastLineHeader)
	While 1
		;next line
		$line = FileReadLine($file)
		If @error = -1 Then ExitLoop	;no more lines
		If $line == "" Then
			; empty line, after the header this means a new condition def starts next line
			$lastLineEmpty = True
		ElseIf $lastLineEmpty == True Then
			;current line should contain a condition name
			$nEntries += 1
			ReDim $conditions[$nEntries]
			_DebugOut ("[readConditionsFromPRT] found condition: "&$line)
			$conditions[$nEntries-1] = $line
			$lastLineEmpty = False
		EndIf
	Wend
	
	If $nEntries == 0 Then
		_DebugOut("[readConditionsFromPRT] {-2} No entries found.")
		SetError(-2)
		Return $conditions
	EndIf
	
	FileClose ( $file )
	_DebugOut ("[readConditionsFromPRT] exit...")
	Return $conditions
EndFunc

Func readPRTfromVTC($vtc)
	Local $prt
	Local $file
	Local $readingPRT
	Local $char
	_DebugOut("[readPRTfromVTC] reading: "&$vtc)	
	$file = FileOpen($vtc,0)
	FileRead ( $file, 2 )
	While 1
		$char = FileRead ( $file, 1 )
		If @error = -1 Then ExitLoop
		If Asc($char) == 0 Then
			If $readingPRT Then
				ExitLoop
			Else
				$readingPRT = True
			EndIf
		ElseIf $readingPRT Then
			$prt = $prt & $char
		EndIf		
	Wend
	FileClose($file)
	_DebugOut("[readPRTfromVTC] result: "&$prt)
	Return $prt
EndFunc

Func readNumberFromVOI($voi)
	Local $number = -1
	Local $file
	Local $line
	Local $entry

	_DebugOut("[readNumberFromVOI] reading: "&$voi)	
	$file = FileOpen($voi,0)

	While 1
		$line = FileReadLine($file)
		If @error = -1 Then ExitLoop
		$entry = StringRegExp($line, "NrOfVOIs:\s*([0-9]+)", 3)			
		If @error == 0 Then
			$number = Int($entry[0])
			ExitLoop
		EndIf		
	Wend
	FileClose($file)
	If $number == -1 Then
		_DebugOut("[readNumberFromVOI] {-1} Number of VOIs not found in .voi file.")
		SetError(-2)
	ElseIf $number == 0 Then
		_DebugOut("[readNumberFromVOI] {-2} VOI contains zero regions.")
		SetError(-2)
	EndIf
	_DebugOut("[readNumberFromVOI] result: "&$number)
	Return $number
EndFunc

Func fileFindName($path,$extension)
	Local $name
	Local $searchHandle

	FileChangeDir ($path)
	$searchHandle = FileFindFirstFile("*."&$extension)
	If $searchHandle = -1 Then
		;no file found
		SetError(1)
		$name = ""
	Else
		$name = FileFindNextFile($searchHandle)		
		If @error Then
			SetError(1)			
		EndIF
	EndIf
	FileClose($searchHandle)

	Return $name
EndFunc

Func filesFindNames($path,$extension)
	Local $name = ""
	Local $names[1]
	Local $searchHandle

	FileChangeDir ($path)
	$searchHandle = FileFindFirstFile("*."&$extension)
	If $searchHandle = -1 Then
		;no file found
		SetError(1)
		Return False
	Else	
		$name = FileFindNextFile($searchHandle)	
		If @error Then
			SetError(1)	
			Return False			
		EndIf			
		While @error == 0			
			_ArrayAdd($names, $name) 			
			$name = FileFindNextFile($searchHandle)			
		Wend
		_ArrayDelete($names, 0)

		
	EndIf
	FileClose($searchHandle)

	Return $names
EndFunc

Func pathGetFileName($path)
	Dim $szDrive, $szDir, $szFName, $szExt
	_PathSplit($path, $szDrive, $szDir, $szFName, $szExt)
	Return $szFName
EndFunc

;Checks the "Global Tools window, Log Tab" to see if the last line ends with asterisks, indicating the GCM plugin is done.
Func pollLogPageGCMfinished()
	Const $ctlLogPage = "[CLASS:QWidget; INSTANCE:12]"
	Const $winGlobalTools = "Global Tools"
	Const $mnuContextLogSelectAllPos = 2
	Const $mnuContextLogCopyPos = 1
	
	Local $log
	Local $finished = 0
	ClipPut("")	;clear clipboard
	WinActivate ( $winGlobalTools )
	;select all
	ControlClick ( $winGlobalTools, "", $ctlLogPage, "secondary")
	Send("+{DOWN " & $mnuContextLogSelectAllPos & "}")
	Send("{ENTER}")
	Sleep(100)
	;copy (to clipboard)
	ControlClick ( $winGlobalTools, "", $ctlLogPage, "secondary")
	Send("+{DOWN " & $mnuContextLogCopyPos & "}")
	Send("{ENTER}")
	;retrieve clipboard text; assume log
	Sleep(100)
	$log = ClipGet ()
	;MsgBox(0,"test", "error = "&@error&" log= "&$log)
	$finished = StringRegExp($log,"(\*+\s*)\Z",0) AND @error ==0
	
	;MsgBox(4096,"test", "error = "&@error&" finished= "&String($finished)&" log= "&$log,10)
	;MsgBox(4096,"test", "error = "&@error&" finished= "&String($finished),1)
	return $finished
EndFunc
Func getDateTimeString()
	Local $dateTime = ""
	$dateTime = String(_DateTimeFormat( _NowCalc(),2))
	$dateTime &= "_"
	$dateTime &= String(_DateTimeFormat( _NowCalc(),4))
	$datetime = StringReplace($datetime, " ", "_")
	$datetime = StringReplace($datetime, ":", "-")
	return $dateTime
EndFunc

